/*********************************************
* vim: set sw=8 ts=8 si et :
* Author: Guido Socher, Copyright: BSD 2 license
*
* Copyright (c) 2014, Guido Socher
* All rights reserved.
* 
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 
* 1. Redistributions of source code must retain the above copyright notice,
* this list of conditions and the following disclaimer.
* 
* 2. Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* This is a an example the 8x1 LCD module from tuxgraphics
* showing how to do analog measurement on two ADC channels.
*
* See http://www.tuxgraphics.org/electronics/
* 
* Chip type           : ATMEGA328p
* Clock frequency     : Internal clock 8 Mhz 
*********************************************/
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#define F_CPU 8000000UL  // 8 MHz
#include <util/delay.h>
#include <stdlib.h> 
#include <string.h> 
#include <avr/eeprom.h> 
#include "lcd.h"
#include "analog.h"

//
#define STR_BUFFER_SIZE 15
static char gStrbuf[STR_BUFFER_SIZE+1];
//
//
void delay_ms(unsigned char ms)
{
        // if we would use the library function _delay_ms
        // everywhere then the code would become very fat
        // We just need integers therefore we wrap it.
        while(ms){
                _delay_ms(0.96);
                ms--;
        }
}



int main(void)
{
        uint16_t tmp;
        uint8_t channel=0; // the adc channnel 0 for ADC0, 1 for ADC1
        // set the clock speed to "no pre-scaler" (8MHz with internal osc or 
        // full external speed)
        // set the clock prescaler. First write CLKPCE to enable setting 
        // of clock the next four instructions.
        // Note that the CKDIV8 Fuse determines the initial
        // value of the CKKPS bits.
        CLKPR=(1<<CLKPCE); // change enable
        CLKPR=0; // "no pre-scaler"
        _delay_loop_1(0); // 60us

        lcd_init();
        lcd_reset();
        sei();
        adc_isr_on(); // analog measurements, start
        lcd_clrscr();
        while (1) {
                lcd_clrscr();
                if (channel >1){
                        channel=0;
                }
                itoa(channel,gStrbuf,10);  
                lcd_puts_p(PSTR("ch"));
                lcd_puts(gStrbuf);

                lcd_puts_p(PSTR(":"));
                tmp=getanalogresult(channel); // take measurement, 12 bit resolution
                // 0V corresponds to 0 on the display
                // and 1.1V corresponds to 1023*4= 4092 
                // That is: because we get the 12 bit via oversampling
                // we achieve 4092 instead of 4096-1. 
                ultoa(tmp,gStrbuf,10);  // convert integer digits to ascii
                lcd_puts(gStrbuf); // print to display
                channel++;
                delay_ms(200);
                delay_ms(200);
                delay_ms(200);
                delay_ms(200);
                delay_ms(200);
        }
        return(0);
}

