/*********************************************
* vim: set sw=8 ts=8 si et :
* Author: Guido Socher, Copyright: BSD 2 license
*
* Copyright (c) 2014, Guido Socher
* All rights reserved.
* 
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 
* 1. Redistributions of source code must retain the above copyright notice,
* this list of conditions and the following disclaimer.
* 
* 2. Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* This is a test program from the 8x1 LCD module from tuxgraphics
* The program will dispaly "OK", "great" in the dispaly and the two
* words will appeart in alternating order in the display.
* 
* See http://www.tuxgraphics.org/electronics/
* 
* Chip type           : ATMEGA328p
* Clock frequency     : Internal clock 8 Mhz 
*********************************************/
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#define F_CPU 8000000UL  // 8 MHz
#include <util/delay.h>
#include <stdlib.h> 
#include <string.h> 
#include <avr/eeprom.h> 
#include "lcd.h"

//debug LED:
// set output to VCC, red LED off
#define LEDOFF PORTD|=(1<<PORTD0)
// set output to GND, red LED on
#define LEDON PORTD&=~(1<<PORTD0)
// to test the state of the LED
#define LEDISOFF PORTD&(1<<PORTD0)
//
void delay_ms(unsigned char ms)
{
        // if we would use the library function _delay_ms
        // everywhere then the code would become very fat
        // We just need integers therefore we wrap it.
        while(ms){
                _delay_ms(0.96);
                ms--;
        }
}

int main(void)
{
        uint8_t toggle=0;
        char great_str[]="great";
        // set the clock speed to "no pre-scaler" (8MHz with internal osc or 
        // full external speed)
        // set the clock prescaler. First write CLKPCE to enable setting 
        // of clock the next four instructions.
        // Note that the CKDIV8 Fuse determines the initial
        // value of the CKKPS bits.
        CLKPR=(1<<CLKPCE); // change enable
        CLKPR=0; // "no pre-scaler"
        _delay_loop_1(0); // 60us

        // debug led, you can not have an LED if you use the uart
        DDRD|= (1<<DDD0); // LED, enable PD0, LED as output
        LEDOFF;
        lcd_init();
        lcd_reset();
        sei();
        //lcd_gotoxy(0,1);
        while (1) {
                lcd_clrscr();
                lcd_home();
                if (toggle){
                        toggle=0;
                        lcd_puts_p(PSTR("OK"));
                }else{
                        toggle=1;
                        lcd_puts(great_str);
                }
                delay_ms(200);
                delay_ms(200);
                delay_ms(200);
                delay_ms(200);
                delay_ms(200);
        }
        return(0);
}

